/**********************************************************************
 * These are functions that allow you to step through the database
 * names on a Notes server. On your first call to GetNextDbName,
 * a list of databases is created and stored, and subsequent calls
 * to the function using the same server and directory name will
 * grab the next database name, until there are no more databases
 * to get. Subdirectories are recursively searched.
 *
 * If you want to get the local databases, use "" as the serverName
 * parameter in the GetNextDbName function. Also, if you want to
 * start your search from the Notes data directory, use "" as the
 * dirName parameter (otherwise the dirName should be relative to the
 * Notes data directory).
 *
 * The database names are stored in a list (the size of which is
 * determined by MAXLIST in the header), and if you get to the end
 * of the list while there are still databases to get, the list gets
 * repopulated with the next set of database names. This buffering
 * of the names should make the functions more efficient, because you
 * won't have to make another call to NSFSearch every time you need
 * to get the next name. However, because of the way the buffer list
 * gets repopulated if there are more than MAXLIST entries, you can
 * only expect good results by going forward through the list (in
 * other words, you shouldn't try to go backwards).
 *
 * A lot of the NSFSearch handling routines are a modification of code 
 * from admin\find_dbs.c in the Notes C-API samples. Otherwise, all of
 * this was written by Julian Robichaux (http://www.nsftools.com).
 * Please see the GetDbList.h header file for more information about
 * the use of the functions below.
 **********************************************************************/


#include <stdio.h>
#include <string.h>

#include <global.h>
#include <nsfdb.h>
#include <nsfsearc.h>
#include <osfile.h>
#include <names.h>
#include <nsferr.h>
#include <textlist.h>
#include <ods.h>
#include <odstypes.h>

#include "getdblist.h"


/* globals */
char	lastServerName[MAXPATH] = "ThisIsNotAServerName";
char	lastDirName[MAXPATH] = "ThisIsNotADirectory";
int	lastFile = 0;
int	fileCount = 0;
BOOL	doneSearching = FALSE;
long	counter = 0;
long	fnum = 0;
FILE_LIST  flist[MAXLIST];	/* the FILE_LIST structure is defined in the header file */

/* forward declarations, not in .h file */
STATUS GetList (char *serverName, char *dirName);
STATUS LNPUBLIC AddToList (void far * pServerName, SEARCH_MATCH far * pSearchMatch,
	ITEM_TABLE far * summary_info);
STATUS LNPUBLIC ProcessSummaryInfo (ITEM_TABLE *summary, char *serverName);



BOOL GetNextDbName (char *serverName, char *dirName, char *retDbName, STATUS *retError)
{
	*retError = NOERROR;

	// if this server or directory name isn't the same as the last one,
	// create a new list to use
	if ((strcmp(serverName, lastServerName)) || (strcmp(dirName, lastDirName)))
	{
		// "clear" existing list
		fileCount = 0;
		lastFile = 0;
		fnum = 0;
		*retError = GetList(serverName, dirName);
		strcpy(lastServerName, serverName);
		strcpy(lastDirName, dirName);
	}

	// if we're at the end of our list, either get more files
	// or (if we're done searching) return FALSE
	while ((fileCount == 0) || (lastFile >= fileCount))
	{
		if (doneSearching)
		{
			retDbName[0] = '\0';
			return FALSE;
		}  else  {
			fileCount = 0;
			lastFile = 0;
			//printf("  ***Getting new list***\n");
			*retError = GetList(serverName, dirName);
		}
	}

	strcpy(retDbName, flist[lastFile].fullPath);
	lastFile++;
	return TRUE;

}



void GetDbNameOnly (char *retDbName)
{
	if (lastFile == 0)
		retDbName[0] = '\0';
	else if ((fileCount == 0) || (lastFile > fileCount))
		retDbName[0] = '\0';
	else
		strcpy(retDbName, flist[lastFile - 1].fileName);

}



void GetDbDiskPath (char *retDbPath)
{
	if (lastFile == 0)
		retDbPath[0] = '\0';
	else if ((fileCount == 0) || (lastFile > fileCount))
		retDbPath[0] = '\0';
	else
		strcpy(retDbPath, flist[lastFile - 1].physicalPath);
}



STATUS GetList (char *serverName, char *dirName)
{
	STATUS error = NOERROR;
	DBHANDLE dir_handle;
	char fullPath[MAXPATH];

	/* Create a fully qualified path */
	if (error = OSPathNetConstruct(NULL, serverName, dirName, fullPath))
	{
		doneSearching = TRUE;
		return error;
	}

	/* Open the directory */
	if (error = NSFDbOpen (fullPath, &dir_handle))
	{
		doneSearching = TRUE;
		return error;
	}

	/* Reset the counter */
	counter = 0;

	/* Call NSFSearch to find files in the directory. For each file found, 
	call an action routine. */
	if (error = NSFSearch (
		dir_handle,        /* directory handle */
		NULLHANDLE,        /* selection formula */
		NULL,              /* title of view in formula */
		SEARCH_FILETYPE +  /* search for files */
		SEARCH_SUMMARY,    /* return a summary buffer */
		FILE_DBDESIGN +    /* find any databases or templates */
		FILE_RECURSE +     /* recurse subdirectories */
		FILE_NOUPDIRS,     /* don't find the ".." dir */
		NULL,              /* starting date */
		AddToList,         /* call for each file found */
		serverName,       /* argument to action routine */
		NULL))             /* returned ending date (unused) */

        {
			NSFDbClose (dir_handle);
			doneSearching = TRUE;
			return error;
        }

	/* Close the directory. */
	error = NSFDbClose (dir_handle);

	/* if we reached the end of the list, set the flag */
	if ((fileCount <= MAXLIST) || (counter == fnum))
		doneSearching = TRUE;

	return (error);

}


STATUS LNPUBLIC AddToList (void far * pServerName, SEARCH_MATCH far *pSearchMatch,
	ITEM_TABLE far *summary_info)
{
	SEARCH_MATCH SearchMatch;
	char         *serverName;

	/* skip this one if the counter isn't high enough, or we've reached
	   the end of the list */
	counter++;
	if ((counter <= fnum) || (fileCount > MAXLIST))
		return NOERROR;

	memcpy( (char*)&SearchMatch, (char*)pSearchMatch, sizeof(SEARCH_MATCH) );

	/*  if (SearchMatch.MatchesFormula != SE_FMATCH)      V3 */
	if (!(SearchMatch.SERetFlags & SE_FMATCH))     /* V4 */
		return (NOERROR);

	serverName = ((char far *)pServerName);

	ProcessSummaryInfo(summary_info, serverName);
	return (NOERROR);
}



STATUS LNPUBLIC ProcessSummaryInfo (ITEM_TABLE *summary, char *serverName)
{
/* All this stuff is a modification of code from admin\find_dbs.c
   in the Notes API samples. */
#define     MAX_ITEMS           20
#define     MAX_TEXT_LEN        100
#define     DATATYPE_SIZE       sizeof(USHORT)
#define     NAME_LENGTH_SIZE    sizeof(USHORT)
#define     ITEM_LENGTH_SIZE    sizeof(USHORT)
#define     NUMERIC_SIZE        sizeof(FLOAT)
#define     TIME_SIZE           ODSLength(_TIMEDATE)


	BYTE       *summary_position;       /* current position in summary */
	ITEM_TABLE item_table;              /* header at start of summary */
	USHORT     item_count;              /* number of items in summary */
	USHORT     name_length[MAX_ITEMS];  /* length of each item name */
	USHORT     item_length[MAX_ITEMS];  /* length of each item */
	char       item_name[MAX_TEXT_LEN]; /* name of a summary item */
	//char       item_text[MAX_TEXT_LEN]; /* text of a summary item */
	char       title[MAX_TEXT_LEN] = "";
	char       path[MAX_TEXT_LEN] = "";
	char       dirOrFile[MAX_TEXT_LEN] = "";
	char       physicalPath[MAX_TEXT_LEN] = "";
	USHORT     i;                       /* a counter */

	/* Get the header at the beginning of the summary buffer. */
	item_table = *summary;

	/* Keep track of where we are in the buffer. */
	summary_position = (BYTE *) summary;

	/* Get the number of items in the summary. */
	item_count = item_table.Items;

	/* Advance the buffer pointer over the header. */
	summary_position += ODSLength(_ITEM_TABLE);

	/* Get the length of each item in the summary. */
	for (i=0; i < item_count; i++)
	{
		memcpy (&name_length[i], summary_position, NAME_LENGTH_SIZE);
		summary_position += NAME_LENGTH_SIZE;
		memcpy (&item_length[i], summary_position, ITEM_LENGTH_SIZE);
		summary_position += ITEM_LENGTH_SIZE;
	}

	/* Start a loop that extracts each item in the summary. */
	for (i=0; i < item_count; i++)
	{
		/* Get the name of the item. */
		memcpy (item_name, summary_position, name_length[i]);
		item_name[name_length[i]] = '\0';
		summary_position += name_length[i];

		/* We don't care about the datatype here, because we know
		   that the datatype of all the items we want is TEXT. */
		summary_position += DATATYPE_SIZE;

		/* Get the data items we care about */
		if (strcmp(item_name, "$TITLE") == 0)
		{
			memcpy (title, summary_position, item_length[i] - DATATYPE_SIZE);
			title[item_length[i] - DATATYPE_SIZE] = '\0';
		}

		if (strcmp(item_name, "$Path") == 0)
		{
			memcpy (path, summary_position, item_length[i] - DATATYPE_SIZE);
			path[item_length[i] - DATATYPE_SIZE] = '\0';
		}

		if (strcmp(item_name, "$Type") == 0)
		{
			// $NOTEFILE for Notes databases, $DIR for directories
			memcpy (dirOrFile, summary_position, item_length[i] - DATATYPE_SIZE);
			dirOrFile[item_length[i] - DATATYPE_SIZE] = '\0';
		}

		if (strcmp(item_name, "$PHYSICALPATH") == 0)
		{
			memcpy (physicalPath, summary_position, item_length[i] - DATATYPE_SIZE);
			physicalPath[item_length[i] - DATATYPE_SIZE] = '\0';
		}

		/* Advance to next item in the summary. */
		summary_position += (item_length[i] - DATATYPE_SIZE);

	/* End of loop that is extracting each item in the summary. */
        }

	/* Add to the file list, unless it's a directory. */
	if (strcmp(dirOrFile, "$DIR") != 0)
	{
		strcpy(flist[fileCount].serverName, serverName);
		strcpy(flist[fileCount].fileName, title);
		strcpy(flist[fileCount].fullPath, path);
		strcpy(flist[fileCount].physicalPath, physicalPath);
		fileCount++;

		/* If we filled up our list, mark the counter position */
		if (fileCount > MAXLIST)
			fnum = counter;
	}

	return (NOERROR);

}


